package io.syndesis.server.api.generator.soap;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.MoreObjects;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import com.google.errorprone.annotations.Var;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.common.model.Violation;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import javax.annotation.CheckReturnValue;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import javax.wsdl.Definition;
import javax.xml.namespace.QName;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link SoapApiModelInfo}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new SoapApiModelInfo.Builder()}.
 */
@Generated(from = "SoapApiModelInfo", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.processing.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
@CheckReturnValue
public final class ImmutableSoapApiModelInfo implements SoapApiModelInfo {
  private final @Nullable String wsdlURL;
  private final InputStream specification;
  private final @Nullable Definition model;
  private final @Nullable QName defaultService;
  private final @Nullable String defaultPort;
  private final @Nullable String defaultAddress;
  private final ImmutableList<QName> services;
  private final ImmutableMap<QName, List<String>> ports;
  private final ImmutableMap<String, String> addresses;
  private final ImmutableList<Violation> warnings;
  private final ImmutableList<Violation> errors;

  private ImmutableSoapApiModelInfo(ImmutableSoapApiModelInfo.Builder builder) {
    this.wsdlURL = builder.wsdlURL;
    this.specification = builder.specification;
    this.model = builder.model;
    this.defaultService = builder.defaultService;
    this.defaultPort = builder.defaultPort;
    this.defaultAddress = builder.defaultAddress;
    if (builder.servicesIsSet()) {
      initShim.services(builder.services.build());
    }
    if (builder.portsIsSet()) {
      initShim.ports(builder.ports.build());
    }
    if (builder.addressesIsSet()) {
      initShim.addresses(builder.addresses.build());
    }
    if (builder.warningsIsSet()) {
      initShim.warnings(builder.warnings.build());
    }
    if (builder.errorsIsSet()) {
      initShim.errors(builder.errors.build());
    }
    this.services = initShim.getServices();
    this.ports = initShim.getPorts();
    this.addresses = initShim.getAddresses();
    this.warnings = initShim.getWarnings();
    this.errors = initShim.getErrors();
    this.initShim = null;
  }

  private ImmutableSoapApiModelInfo(
      @Nullable String wsdlURL,
      InputStream specification,
      @Nullable Definition model,
      @Nullable QName defaultService,
      @Nullable String defaultPort,
      @Nullable String defaultAddress,
      ImmutableList<QName> services,
      ImmutableMap<QName, List<String>> ports,
      ImmutableMap<String, String> addresses,
      ImmutableList<Violation> warnings,
      ImmutableList<Violation> errors) {
    this.wsdlURL = wsdlURL;
    this.specification = specification;
    this.model = model;
    this.defaultService = defaultService;
    this.defaultPort = defaultPort;
    this.defaultAddress = defaultAddress;
    this.services = services;
    this.ports = ports;
    this.addresses = addresses;
    this.warnings = warnings;
    this.errors = errors;
    this.initShim = null;
  }

  private static final byte STAGE_INITIALIZING = -1;
  private static final byte STAGE_UNINITIALIZED = 0;
  private static final byte STAGE_INITIALIZED = 1;
  @SuppressWarnings("Immutable")
  private transient volatile InitShim initShim = new InitShim();

  @Generated(from = "SoapApiModelInfo", generator = "Immutables")
  private final class InitShim {
    private byte servicesBuildStage = STAGE_UNINITIALIZED;
    private ImmutableList<QName> services;

    ImmutableList<QName> getServices() {
      if (servicesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (servicesBuildStage == STAGE_UNINITIALIZED) {
        servicesBuildStage = STAGE_INITIALIZING;
        this.services = ImmutableList.copyOf(getServicesInitialize());
        servicesBuildStage = STAGE_INITIALIZED;
      }
      return this.services;
    }

    void services(ImmutableList<QName> services) {
      this.services = services;
      servicesBuildStage = STAGE_INITIALIZED;
    }

    private byte portsBuildStage = STAGE_UNINITIALIZED;
    private ImmutableMap<QName, List<String>> ports;

    ImmutableMap<QName, List<String>> getPorts() {
      if (portsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (portsBuildStage == STAGE_UNINITIALIZED) {
        portsBuildStage = STAGE_INITIALIZING;
        this.ports = ImmutableMap.copyOf(getPortsInitialize());
        portsBuildStage = STAGE_INITIALIZED;
      }
      return this.ports;
    }

    void ports(ImmutableMap<QName, List<String>> ports) {
      this.ports = ports;
      portsBuildStage = STAGE_INITIALIZED;
    }

    private byte addressesBuildStage = STAGE_UNINITIALIZED;
    private ImmutableMap<String, String> addresses;

    ImmutableMap<String, String> getAddresses() {
      if (addressesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (addressesBuildStage == STAGE_UNINITIALIZED) {
        addressesBuildStage = STAGE_INITIALIZING;
        this.addresses = ImmutableMap.copyOf(getAddressesInitialize());
        addressesBuildStage = STAGE_INITIALIZED;
      }
      return this.addresses;
    }

    void addresses(ImmutableMap<String, String> addresses) {
      this.addresses = addresses;
      addressesBuildStage = STAGE_INITIALIZED;
    }

    private byte warningsBuildStage = STAGE_UNINITIALIZED;
    private ImmutableList<Violation> warnings;

    ImmutableList<Violation> getWarnings() {
      if (warningsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (warningsBuildStage == STAGE_UNINITIALIZED) {
        warningsBuildStage = STAGE_INITIALIZING;
        this.warnings = ImmutableList.copyOf(getWarningsInitialize());
        warningsBuildStage = STAGE_INITIALIZED;
      }
      return this.warnings;
    }

    void warnings(ImmutableList<Violation> warnings) {
      this.warnings = warnings;
      warningsBuildStage = STAGE_INITIALIZED;
    }

    private byte errorsBuildStage = STAGE_UNINITIALIZED;
    private ImmutableList<Violation> errors;

    ImmutableList<Violation> getErrors() {
      if (errorsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (errorsBuildStage == STAGE_UNINITIALIZED) {
        errorsBuildStage = STAGE_INITIALIZING;
        this.errors = ImmutableList.copyOf(getErrorsInitialize());
        errorsBuildStage = STAGE_INITIALIZED;
      }
      return this.errors;
    }

    void errors(ImmutableList<Violation> errors) {
      this.errors = errors;
      errorsBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      List<String> attributes = new ArrayList<>();
      if (servicesBuildStage == STAGE_INITIALIZING) attributes.add("services");
      if (portsBuildStage == STAGE_INITIALIZING) attributes.add("ports");
      if (addressesBuildStage == STAGE_INITIALIZING) attributes.add("addresses");
      if (warningsBuildStage == STAGE_INITIALIZING) attributes.add("warnings");
      if (errorsBuildStage == STAGE_INITIALIZING) attributes.add("errors");
      return "Cannot build SoapApiModelInfo, attribute initializers form cycle " + attributes;
    }
  }

  private List<QName> getServicesInitialize() {
    return SoapApiModelInfo.super.getServices();
  }

  private Map<QName, List<String>> getPortsInitialize() {
    return SoapApiModelInfo.super.getPorts();
  }

  private Map<String, String> getAddressesInitialize() {
    return SoapApiModelInfo.super.getAddresses();
  }

  private List<Violation> getWarningsInitialize() {
    return SoapApiModelInfo.super.getWarnings();
  }

  private List<Violation> getErrorsInitialize() {
    return SoapApiModelInfo.super.getErrors();
  }

  /**
   * @return The value of the {@code wsdlURL} attribute
   */
  @JsonProperty("wsdlURL")
  @Override
  public Optional<String> getWsdlURL() {
    return Optional.ofNullable(wsdlURL);
  }

  /**
   * @return The value of the {@code specification} attribute
   */
  @JsonProperty("specification")
  @Override
  public InputStream getSpecification() {
    return specification;
  }

  /**
   * @return The value of the {@code model} attribute
   */
  @JsonProperty("model")
  @Override
  public Optional<Definition> getModel() {
    return Optional.ofNullable(model);
  }

  /**
   * @return The value of the {@code defaultService} attribute
   */
  @JsonProperty("defaultService")
  @Override
  public Optional<QName> getDefaultService() {
    return Optional.ofNullable(defaultService);
  }

  /**
   * @return The value of the {@code defaultPort} attribute
   */
  @JsonProperty("defaultPort")
  @Override
  public Optional<String> getDefaultPort() {
    return Optional.ofNullable(defaultPort);
  }

  /**
   * @return The value of the {@code defaultAddress} attribute
   */
  @JsonProperty("defaultAddress")
  @Override
  public Optional<String> getDefaultAddress() {
    return Optional.ofNullable(defaultAddress);
  }

  /**
   * @return The value of the {@code services} attribute
   */
  @JsonProperty("services")
  @Override
  public ImmutableList<QName> getServices() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getServices()
        : this.services;
  }

  /**
   * @return The value of the {@code ports} attribute
   */
  @JsonProperty("ports")
  @Override
  public ImmutableMap<QName, List<String>> getPorts() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getPorts()
        : this.ports;
  }

  /**
   * @return The value of the {@code addresses} attribute
   */
  @JsonProperty("addresses")
  @Override
  public ImmutableMap<String, String> getAddresses() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getAddresses()
        : this.addresses;
  }

  /**
   * @return The value of the {@code warnings} attribute
   */
  @JsonProperty("warnings")
  @Override
  public ImmutableList<Violation> getWarnings() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getWarnings()
        : this.warnings;
  }

  /**
   * @return The value of the {@code errors} attribute
   */
  @JsonProperty("errors")
  @Override
  public ImmutableList<Violation> getErrors() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getErrors()
        : this.errors;
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link SoapApiModelInfo#getWsdlURL() wsdlURL} attribute.
   * @param value The value for wsdlURL
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withWsdlURL(String value) {
    @Nullable String newValue = Objects.requireNonNull(value, "wsdlURL");
    if (Objects.equals(this.wsdlURL, newValue)) return this;
    return new ImmutableSoapApiModelInfo(
        newValue,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link SoapApiModelInfo#getWsdlURL() wsdlURL} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for wsdlURL
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withWsdlURL(Optional<String> optional) {
    @Nullable String value = optional.orElse(null);
    if (Objects.equals(this.wsdlURL, value)) return this;
    return new ImmutableSoapApiModelInfo(
        value,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link SoapApiModelInfo#getSpecification() specification} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for specification
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableSoapApiModelInfo withSpecification(InputStream value) {
    if (this.specification == value) return this;
    InputStream newValue = Objects.requireNonNull(value, "specification");
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        newValue,
        this.model,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link SoapApiModelInfo#getModel() model} attribute.
   * @param value The value for model
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withModel(Definition value) {
    @Nullable Definition newValue = Objects.requireNonNull(value, "model");
    if (this.model == newValue) return this;
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        newValue,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link SoapApiModelInfo#getModel() model} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for model
   * @return A modified copy of {@code this} object
   */
  @SuppressWarnings("unchecked") // safe covariant cast
  public final ImmutableSoapApiModelInfo withModel(Optional<? extends Definition> optional) {
    @Nullable Definition value = optional.orElse(null);
    if (this.model == value) return this;
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        value,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link SoapApiModelInfo#getDefaultService() defaultService} attribute.
   * @param value The value for defaultService
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withDefaultService(QName value) {
    @Nullable QName newValue = Objects.requireNonNull(value, "defaultService");
    if (this.defaultService == newValue) return this;
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        newValue,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link SoapApiModelInfo#getDefaultService() defaultService} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for defaultService
   * @return A modified copy of {@code this} object
   */
  @SuppressWarnings("unchecked") // safe covariant cast
  public final ImmutableSoapApiModelInfo withDefaultService(Optional<? extends QName> optional) {
    @Nullable QName value = optional.orElse(null);
    if (this.defaultService == value) return this;
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        value,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link SoapApiModelInfo#getDefaultPort() defaultPort} attribute.
   * @param value The value for defaultPort
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withDefaultPort(String value) {
    @Nullable String newValue = Objects.requireNonNull(value, "defaultPort");
    if (Objects.equals(this.defaultPort, newValue)) return this;
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        newValue,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link SoapApiModelInfo#getDefaultPort() defaultPort} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for defaultPort
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withDefaultPort(Optional<String> optional) {
    @Nullable String value = optional.orElse(null);
    if (Objects.equals(this.defaultPort, value)) return this;
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        value,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link SoapApiModelInfo#getDefaultAddress() defaultAddress} attribute.
   * @param value The value for defaultAddress
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withDefaultAddress(String value) {
    @Nullable String newValue = Objects.requireNonNull(value, "defaultAddress");
    if (Objects.equals(this.defaultAddress, newValue)) return this;
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        newValue,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link SoapApiModelInfo#getDefaultAddress() defaultAddress} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for defaultAddress
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withDefaultAddress(Optional<String> optional) {
    @Nullable String value = optional.orElse(null);
    if (Objects.equals(this.defaultAddress, value)) return this;
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        value,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link SoapApiModelInfo#getServices() services}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withServices(QName... elements) {
    ImmutableList<QName> newValue = ImmutableList.copyOf(elements);
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        newValue,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link SoapApiModelInfo#getServices() services}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of services elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withServices(Iterable<? extends QName> elements) {
    if (this.services == elements) return this;
    ImmutableList<QName> newValue = ImmutableList.copyOf(elements);
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        newValue,
        this.ports,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by replacing the {@link SoapApiModelInfo#getPorts() ports} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the ports map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withPorts(Map<? extends QName, ? extends List<String>> entries) {
    if (this.ports == entries) return this;
    ImmutableMap<QName, List<String>> newValue = ImmutableMap.copyOf(entries);
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        newValue,
        this.addresses,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object by replacing the {@link SoapApiModelInfo#getAddresses() addresses} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the addresses map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withAddresses(Map<String, ? extends String> entries) {
    if (this.addresses == entries) return this;
    ImmutableMap<String, String> newValue = ImmutableMap.copyOf(entries);
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        newValue,
        this.warnings,
        this.errors);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link SoapApiModelInfo#getWarnings() warnings}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withWarnings(Violation... elements) {
    ImmutableList<Violation> newValue = ImmutableList.copyOf(elements);
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        newValue,
        this.errors);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link SoapApiModelInfo#getWarnings() warnings}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of warnings elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withWarnings(Iterable<? extends Violation> elements) {
    if (this.warnings == elements) return this;
    ImmutableList<Violation> newValue = ImmutableList.copyOf(elements);
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        newValue,
        this.errors);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link SoapApiModelInfo#getErrors() errors}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withErrors(Violation... elements) {
    ImmutableList<Violation> newValue = ImmutableList.copyOf(elements);
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        newValue);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link SoapApiModelInfo#getErrors() errors}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of errors elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSoapApiModelInfo withErrors(Iterable<? extends Violation> elements) {
    if (this.errors == elements) return this;
    ImmutableList<Violation> newValue = ImmutableList.copyOf(elements);
    return new ImmutableSoapApiModelInfo(
        this.wsdlURL,
        this.specification,
        this.model,
        this.defaultService,
        this.defaultPort,
        this.defaultAddress,
        this.services,
        this.ports,
        this.addresses,
        this.warnings,
        newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableSoapApiModelInfo} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableSoapApiModelInfo
        && equalTo((ImmutableSoapApiModelInfo) another);
  }

  private boolean equalTo(ImmutableSoapApiModelInfo another) {
    return Objects.equals(wsdlURL, another.wsdlURL)
        && specification.equals(another.specification)
        && Objects.equals(model, another.model)
        && Objects.equals(defaultService, another.defaultService)
        && Objects.equals(defaultPort, another.defaultPort)
        && Objects.equals(defaultAddress, another.defaultAddress)
        && services.equals(another.services)
        && ports.equals(another.ports)
        && addresses.equals(another.addresses)
        && warnings.equals(another.warnings)
        && errors.equals(another.errors);
  }

  /**
   * Computes a hash code from attributes: {@code wsdlURL}, {@code specification}, {@code model}, {@code defaultService}, {@code defaultPort}, {@code defaultAddress}, {@code services}, {@code ports}, {@code addresses}, {@code warnings}, {@code errors}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    @Var int h = 5381;
    h += (h << 5) + Objects.hashCode(wsdlURL);
    h += (h << 5) + specification.hashCode();
    h += (h << 5) + Objects.hashCode(model);
    h += (h << 5) + Objects.hashCode(defaultService);
    h += (h << 5) + Objects.hashCode(defaultPort);
    h += (h << 5) + Objects.hashCode(defaultAddress);
    h += (h << 5) + services.hashCode();
    h += (h << 5) + ports.hashCode();
    h += (h << 5) + addresses.hashCode();
    h += (h << 5) + warnings.hashCode();
    h += (h << 5) + errors.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code SoapApiModelInfo} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("SoapApiModelInfo")
        .omitNullValues()
        .add("wsdlURL", wsdlURL)
        .add("specification", specification)
        .add("model", model)
        .add("defaultService", defaultService)
        .add("defaultPort", defaultPort)
        .add("defaultAddress", defaultAddress)
        .add("services", services)
        .add("ports", ports)
        .add("addresses", addresses)
        .add("warnings", warnings)
        .add("errors", errors)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link SoapApiModelInfo} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable SoapApiModelInfo instance
   */
  public static ImmutableSoapApiModelInfo copyOf(SoapApiModelInfo instance) {
    if (instance instanceof ImmutableSoapApiModelInfo) {
      return (ImmutableSoapApiModelInfo) instance;
    }
    return new SoapApiModelInfo.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableSoapApiModelInfo ImmutableSoapApiModelInfo}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "SoapApiModelInfo", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private static final long INIT_BIT_SPECIFICATION = 0x1L;
    private static final long OPT_BIT_SERVICES = 0x1L;
    private static final long OPT_BIT_PORTS = 0x2L;
    private static final long OPT_BIT_ADDRESSES = 0x4L;
    private static final long OPT_BIT_WARNINGS = 0x8L;
    private static final long OPT_BIT_ERRORS = 0x10L;
    private long initBits = 0x1L;
    private long optBits;

    private @Nullable String wsdlURL;
    private @Nullable InputStream specification;
    private @Nullable Definition model;
    private @Nullable QName defaultService;
    private @Nullable String defaultPort;
    private @Nullable String defaultAddress;
    private ImmutableList.Builder<QName> services = ImmutableList.builder();
    private ImmutableMap.Builder<QName, List<String>> ports = ImmutableMap.builder();
    private ImmutableMap.Builder<String, String> addresses = ImmutableMap.builder();
    private ImmutableList.Builder<Violation> warnings = ImmutableList.builder();
    private ImmutableList.Builder<Violation> errors = ImmutableList.builder();

    /**
     * Creates a builder for {@link ImmutableSoapApiModelInfo ImmutableSoapApiModelInfo} instances.
     * <pre>
     * new SoapApiModelInfo.Builder()
     *    .wsdlURL(String) // optional {@link SoapApiModelInfo#getWsdlURL() wsdlURL}
     *    .specification(java.io.InputStream) // required {@link SoapApiModelInfo#getSpecification() specification}
     *    .model(javax.wsdl.Definition) // optional {@link SoapApiModelInfo#getModel() model}
     *    .defaultService(javax.xml.namespace.QName) // optional {@link SoapApiModelInfo#getDefaultService() defaultService}
     *    .defaultPort(String) // optional {@link SoapApiModelInfo#getDefaultPort() defaultPort}
     *    .defaultAddress(String) // optional {@link SoapApiModelInfo#getDefaultAddress() defaultAddress}
     *    .addServices|addAllServices(javax.xml.namespace.QName) // {@link SoapApiModelInfo#getServices() services} elements
     *    .putPorts|putAllPorts(javax.xml.namespace.QName =&gt; List&amp;lt;String&amp;gt;) // {@link SoapApiModelInfo#getPorts() ports} mappings
     *    .putAddresses|putAllAddresses(String =&gt; String) // {@link SoapApiModelInfo#getAddresses() addresses} mappings
     *    .addWarnings|addAllWarnings(io.syndesis.common.model.Violation) // {@link SoapApiModelInfo#getWarnings() warnings} elements
     *    .addErrors|addAllErrors(io.syndesis.common.model.Violation) // {@link SoapApiModelInfo#getErrors() errors} elements
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof SoapApiModelInfo.Builder)) {
        throw new UnsupportedOperationException("Use: new SoapApiModelInfo.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code SoapApiModelInfo} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder from(SoapApiModelInfo instance) {
      Objects.requireNonNull(instance, "instance");
      Optional<String> wsdlURLOptional = instance.getWsdlURL();
      if (wsdlURLOptional.isPresent()) {
        wsdlURL(wsdlURLOptional);
      }
      specification(instance.getSpecification());
      Optional<Definition> modelOptional = instance.getModel();
      if (modelOptional.isPresent()) {
        model(modelOptional);
      }
      Optional<QName> defaultServiceOptional = instance.getDefaultService();
      if (defaultServiceOptional.isPresent()) {
        defaultService(defaultServiceOptional);
      }
      Optional<String> defaultPortOptional = instance.getDefaultPort();
      if (defaultPortOptional.isPresent()) {
        defaultPort(defaultPortOptional);
      }
      Optional<String> defaultAddressOptional = instance.getDefaultAddress();
      if (defaultAddressOptional.isPresent()) {
        defaultAddress(defaultAddressOptional);
      }
      addAllServices(instance.getServices());
      putAllPorts(instance.getPorts());
      putAllAddresses(instance.getAddresses());
      addAllWarnings(instance.getWarnings());
      addAllErrors(instance.getErrors());
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Initializes the optional value {@link SoapApiModelInfo#getWsdlURL() wsdlURL} to wsdlURL.
     * @param wsdlURL The value for wsdlURL
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder wsdlURL(String wsdlURL) {
      this.wsdlURL = Objects.requireNonNull(wsdlURL, "wsdlURL");
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Initializes the optional value {@link SoapApiModelInfo#getWsdlURL() wsdlURL} to wsdlURL.
     * @param wsdlURL The value for wsdlURL
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("wsdlURL")
    public final SoapApiModelInfo.Builder wsdlURL(Optional<String> wsdlURL) {
      this.wsdlURL = wsdlURL.orElse(null);
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Initializes the value for the {@link SoapApiModelInfo#getSpecification() specification} attribute.
     * @param specification The value for specification 
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("specification")
    public final SoapApiModelInfo.Builder specification(InputStream specification) {
      this.specification = Objects.requireNonNull(specification, "specification");
      initBits &= ~INIT_BIT_SPECIFICATION;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Initializes the optional value {@link SoapApiModelInfo#getModel() model} to model.
     * @param model The value for model
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder model(Definition model) {
      this.model = Objects.requireNonNull(model, "model");
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Initializes the optional value {@link SoapApiModelInfo#getModel() model} to model.
     * @param model The value for model
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("model")
    public final SoapApiModelInfo.Builder model(Optional<? extends Definition> model) {
      this.model = model.orElse(null);
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Initializes the optional value {@link SoapApiModelInfo#getDefaultService() defaultService} to defaultService.
     * @param defaultService The value for defaultService
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder defaultService(QName defaultService) {
      this.defaultService = Objects.requireNonNull(defaultService, "defaultService");
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Initializes the optional value {@link SoapApiModelInfo#getDefaultService() defaultService} to defaultService.
     * @param defaultService The value for defaultService
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("defaultService")
    public final SoapApiModelInfo.Builder defaultService(Optional<? extends QName> defaultService) {
      this.defaultService = defaultService.orElse(null);
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Initializes the optional value {@link SoapApiModelInfo#getDefaultPort() defaultPort} to defaultPort.
     * @param defaultPort The value for defaultPort
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder defaultPort(String defaultPort) {
      this.defaultPort = Objects.requireNonNull(defaultPort, "defaultPort");
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Initializes the optional value {@link SoapApiModelInfo#getDefaultPort() defaultPort} to defaultPort.
     * @param defaultPort The value for defaultPort
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("defaultPort")
    public final SoapApiModelInfo.Builder defaultPort(Optional<String> defaultPort) {
      this.defaultPort = defaultPort.orElse(null);
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Initializes the optional value {@link SoapApiModelInfo#getDefaultAddress() defaultAddress} to defaultAddress.
     * @param defaultAddress The value for defaultAddress
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder defaultAddress(String defaultAddress) {
      this.defaultAddress = Objects.requireNonNull(defaultAddress, "defaultAddress");
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Initializes the optional value {@link SoapApiModelInfo#getDefaultAddress() defaultAddress} to defaultAddress.
     * @param defaultAddress The value for defaultAddress
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("defaultAddress")
    public final SoapApiModelInfo.Builder defaultAddress(Optional<String> defaultAddress) {
      this.defaultAddress = defaultAddress.orElse(null);
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Adds one element to {@link SoapApiModelInfo#getServices() services} list.
     * @param element A services element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder addServices(QName element) {
      this.services.add(element);
      optBits |= OPT_BIT_SERVICES;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Adds elements to {@link SoapApiModelInfo#getServices() services} list.
     * @param elements An array of services elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder addServices(QName... elements) {
      this.services.add(elements);
      optBits |= OPT_BIT_SERVICES;
      return (SoapApiModelInfo.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link SoapApiModelInfo#getServices() services} list.
     * @param elements An iterable of services elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("services")
    public final SoapApiModelInfo.Builder services(Iterable<? extends QName> elements) {
      this.services = ImmutableList.builder();
      return addAllServices(elements);
    }

    /**
     * Adds elements to {@link SoapApiModelInfo#getServices() services} list.
     * @param elements An iterable of services elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder addAllServices(Iterable<? extends QName> elements) {
      this.services.addAll(elements);
      optBits |= OPT_BIT_SERVICES;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Put one entry to the {@link SoapApiModelInfo#getPorts() ports} map.
     * @param key The key in the ports map
     * @param value The associated value in the ports map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder putPorts(QName key, List<String> value) {
      this.ports.put(key, value);
      optBits |= OPT_BIT_PORTS;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Put one entry to the {@link SoapApiModelInfo#getPorts() ports} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder putPorts(Map.Entry<? extends QName, ? extends List<String>> entry) {
      this.ports.put(entry);
      optBits |= OPT_BIT_PORTS;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link SoapApiModelInfo#getPorts() ports} map. Nulls are not permitted
     * @param entries The entries that will be added to the ports map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("ports")
    public final SoapApiModelInfo.Builder ports(Map<? extends QName, ? extends List<String>> entries) {
      this.ports = ImmutableMap.builder();
      optBits |= OPT_BIT_PORTS;
      return putAllPorts(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link SoapApiModelInfo#getPorts() ports} map. Nulls are not permitted
     * @param entries The entries that will be added to the ports map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder putAllPorts(Map<? extends QName, ? extends List<String>> entries) {
      this.ports.putAll(entries);
      optBits |= OPT_BIT_PORTS;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Put one entry to the {@link SoapApiModelInfo#getAddresses() addresses} map.
     * @param key The key in the addresses map
     * @param value The associated value in the addresses map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder putAddresses(String key, String value) {
      this.addresses.put(key, value);
      optBits |= OPT_BIT_ADDRESSES;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Put one entry to the {@link SoapApiModelInfo#getAddresses() addresses} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder putAddresses(Map.Entry<String, ? extends String> entry) {
      this.addresses.put(entry);
      optBits |= OPT_BIT_ADDRESSES;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link SoapApiModelInfo#getAddresses() addresses} map. Nulls are not permitted
     * @param entries The entries that will be added to the addresses map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("addresses")
    public final SoapApiModelInfo.Builder addresses(Map<String, ? extends String> entries) {
      this.addresses = ImmutableMap.builder();
      optBits |= OPT_BIT_ADDRESSES;
      return putAllAddresses(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link SoapApiModelInfo#getAddresses() addresses} map. Nulls are not permitted
     * @param entries The entries that will be added to the addresses map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder putAllAddresses(Map<String, ? extends String> entries) {
      this.addresses.putAll(entries);
      optBits |= OPT_BIT_ADDRESSES;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Adds one element to {@link SoapApiModelInfo#getWarnings() warnings} list.
     * @param element A warnings element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder addWarnings(Violation element) {
      this.warnings.add(element);
      optBits |= OPT_BIT_WARNINGS;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Adds elements to {@link SoapApiModelInfo#getWarnings() warnings} list.
     * @param elements An array of warnings elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder addWarnings(Violation... elements) {
      this.warnings.add(elements);
      optBits |= OPT_BIT_WARNINGS;
      return (SoapApiModelInfo.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link SoapApiModelInfo#getWarnings() warnings} list.
     * @param elements An iterable of warnings elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("warnings")
    public final SoapApiModelInfo.Builder warnings(Iterable<? extends Violation> elements) {
      this.warnings = ImmutableList.builder();
      return addAllWarnings(elements);
    }

    /**
     * Adds elements to {@link SoapApiModelInfo#getWarnings() warnings} list.
     * @param elements An iterable of warnings elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder addAllWarnings(Iterable<? extends Violation> elements) {
      this.warnings.addAll(elements);
      optBits |= OPT_BIT_WARNINGS;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Adds one element to {@link SoapApiModelInfo#getErrors() errors} list.
     * @param element A errors element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder addErrors(Violation element) {
      this.errors.add(element);
      optBits |= OPT_BIT_ERRORS;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Adds elements to {@link SoapApiModelInfo#getErrors() errors} list.
     * @param elements An array of errors elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder addErrors(Violation... elements) {
      this.errors.add(elements);
      optBits |= OPT_BIT_ERRORS;
      return (SoapApiModelInfo.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link SoapApiModelInfo#getErrors() errors} list.
     * @param elements An iterable of errors elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("errors")
    public final SoapApiModelInfo.Builder errors(Iterable<? extends Violation> elements) {
      this.errors = ImmutableList.builder();
      return addAllErrors(elements);
    }

    /**
     * Adds elements to {@link SoapApiModelInfo#getErrors() errors} list.
     * @param elements An iterable of errors elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final SoapApiModelInfo.Builder addAllErrors(Iterable<? extends Violation> elements) {
      this.errors.addAll(elements);
      optBits |= OPT_BIT_ERRORS;
      return (SoapApiModelInfo.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableSoapApiModelInfo ImmutableSoapApiModelInfo}.
     * @return An immutable instance of SoapApiModelInfo
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableSoapApiModelInfo build() {
      if (initBits != 0) {
        throw new IllegalStateException(formatRequiredAttributesMessage());
      }
      return new ImmutableSoapApiModelInfo(this);
    }

    private boolean servicesIsSet() {
      return (optBits & OPT_BIT_SERVICES) != 0;
    }

    private boolean portsIsSet() {
      return (optBits & OPT_BIT_PORTS) != 0;
    }

    private boolean addressesIsSet() {
      return (optBits & OPT_BIT_ADDRESSES) != 0;
    }

    private boolean warningsIsSet() {
      return (optBits & OPT_BIT_WARNINGS) != 0;
    }

    private boolean errorsIsSet() {
      return (optBits & OPT_BIT_ERRORS) != 0;
    }

    private String formatRequiredAttributesMessage() {
      List<String> attributes = new ArrayList<>();
      if ((initBits & INIT_BIT_SPECIFICATION) != 0) attributes.add("specification");
      return "Cannot build SoapApiModelInfo, some of required attributes are not set " + attributes;
    }
  }
}
